#!/usr/bin/perl
# Decoder of The Furry Code
# (c) 2006 Anicka Bernathova <anicka@anicka.net> and Martin Mares <mj@ucw.cz>
# GPL'ed

use strict;
use warnings;
use lib ".";
use Code;

$code_name = "Furry";
$code_type = "FURRY";
$guess_pattern = "^F.*[+-]";

$intro_text = <<EOF ;
<p><a href="http://captainpackrat.com/furry/furcode.htm">The Code of the Furries</a>
is a simple and compact way how to tell the others about your furry personality.
It's similar in spirit to the <a href="http://www.geekcode.com/">Geek Code</a>.
The Furry Code consists of a series of letters, numbers and symbols, each
describing you in some category. Sometimes, it's a little cryptic to the
uninitiated, and here comes our little tool to help you.
EOF

$bot_text = <<EOF ;
<p>This decoder has been written by <a href="http://www.anicka.net/">Anicka</a>
(<a href="http://www.anicka.net/ungeek.php?code=1">Furry Code</a>)
and <a href="http://mj.ucw.cz/">Martin</a> (<a
href="http://mj.ucw.cz/cgi/furry-decoded.cgi">Furry Code</a>) on the Christmas of
2006. If you have any bug reports or ideas on what could be improved, please
<a href="mailto:mj+fc\@ucw.cz">tell us</a>. You can also download
<a href="ftp://atrey.karlin.mff.cuni.cz/pub/local/mj/misc/ungeek-furry-1.1.tar.gz">the source code</a>.
There is also a <a href="/geek/">Geek Code Decoder</a> here.
EOF

sub decode_species($$) {
	my ($c, $val) = @_;
	my ($fam, $spec, $name, $mod, $size, $rel) = ($val =~ /^([A-Z][^0-9A-Zcfhmpstuw]*)([A-Z]*)(\[[^\]]+\])?([a-z]*)(\d*)([a-z]*)$/) or return 0;
	print "SPECIES: fam=$fam spec=$spec mod=$mod size=$size rel=$rel\n" if $debug > 1;
	$mod ne "" && $size eq "" and return 0;
	my %families = (
		"A"	=> "Avians",
		"Ar"	=> "Artiodactyla (two-toed hoofed animals)",
		"C"	=> "Canidae (dog family)",
		"Ce"	=> "Centaurs",
		"Ch"	=> "Chiroptera (bat family)",
		"Ct"	=> "Cetacea (whale family)",
		"D"	=> "Dragons, dinosaurs, and other reptiles",
		"E"	=> "Equidae (horse family)",
		"Ed"	=> "Edentata",
		"F"	=> "Felidae (cat family)",
		"G"	=> "Generic furry",
		"H"	=> "Herpestidae (mongoose family)",
		"Hy"	=> "Hyaenidae (hyena family)",
		"I"	=> "Insectivora",
		"L"	=> "Lagomorpha (rabbit family)",
		"M"	=> "Mustelidae (weasel family)",
		"Ma"	=> "Marsupialia",
		"Mo"	=> "Monotremata",
		"P"	=> "Procyonidae (raccoon family)",
		"Pi"	=> "Pinnipedia (seal family)",
		"Pr"	=> "Primates",
		"R"	=> "Rodentia",
		"U"	=> "Ursidae (bear family)",
		"V"	=> "Viverridae (civet family)",
		"X"	=> "Mythical creatures (other than centaurs and dragons)",
		"Z"	=> "Polymorph (I change shape at will, with no \"normal\" shape)",
	);
	my %species = (
		"AA"	=> "Albatross",
		"AD"	=> "Duck",
		"AE"	=> "Eagle",
		"AF"	=> "Falcon",
		"AG"	=> "Seagull",
		"AH"	=> "Hawk",
		"AK"	=> "Kiwi",
		"AO"	=> "Owl",
		"AR"	=> "Raven",
		"AS"	=> "Sparrow",
		"AW"	=> "Wren",
		"ArA"	=> "Antelope",
		"ArB"	=> "Buffalo",
		"ArC"	=> "Cattle",
		"ArD"	=> "Deer",
		"ArE"	=> "Camel",
		"ArF"	=> "Giraffe",
		"ArG"	=> "Goat",
		"ArL"	=> "Llama",
		"ArM"	=> "Moose",
		"ArN"	=> "Gnu",
		"ArP"	=> "Pig",
		"ArR"	=> "Reindeer",
		"ArS"	=> "Sheep",
		"ArW"	=> "Warthog",
		"ArZ"	=> "Gazelle",
		"CA"	=> "Arctic fox",
		"CC"	=> "Coyote",
		"CD"	=> "Dingo",
		"CF"	=> "Fox",
		"CJ"	=> "Jackal",
		"CM"	=> "Domestic mutt",
		"CW"	=> "Wolf",
		"CX"	=> "Maned wolf",
		"CeH"	=> "Common (horse-human) centaur",
		"CeZ"	=> "Zebra-centaur",
		"ChF"	=> "Flying fox or fruit bat",
		"ChV"	=> "Vampire bat",
		"CtD"	=> "Dolphin",
		"DA"	=> "Alligator or crocodile",
		"DC"	=> "Coelurosaur (Velociraptor etc)",
		"DD"	=> "Dragon",
		"DK"	=> "Carnosaur (Tyrannosaurus etc)",
		"DL"	=> "Lizard",
		"DS"	=> "Snake",
		"DT"	=> "Tortoise or turtle",
		"ED"	=> "Donkey",
		"EH"	=> "Horse",
		"EZ"	=> "Zebra",
		"EdA"	=> "Anteater",
		"EdR"	=> "Armadillo",
		"EdS"	=> "Sloth",
		"FB"	=> "Bobcat",
		"FC"	=> "Clouded leopard",
		"FD"	=> "Domestic cat",
		"FH"	=> "Cheetah",
		"FJ"	=> "Jaguar",
		"FL"	=> "Lion",
		"FM"	=> "Puma/cougar/mountain lion",
		"FO"	=> "Ocelot",
		"FP"	=> "Leopard/panther",
		"FS"	=> "Snow leopard",
		"FT"	=> "Tiger",
		"FV"	=> "Serval",
		"FW"	=> "Wild cat",
		"FX"	=> "Lynx",
		"HK"	=> "Meerkat",
		"HM"	=> "Mongoose",
		"HyH"	=> "Hyena",
		"IH"	=> "Hedgehog",
		"IM"	=> "Mole",
		"LH"	=> "Hare",
		"LJ"	=> "Jackrabbit",
		"LR"	=> "Rabbit",
		"MA"	=> "Sable",
		"MB"	=> "Badger",
		"ME"	=> "Ermine/stoat",
		"MF"	=> "Ferret",
		"MK"	=> "Mink",
		"MM"	=> "Marten",
		"MO"	=> "Otter",
		"MP"	=> "Polecat",
		"MR"	=> "Ratel/honey badger",
		"MS"	=> "Skunk",
		"MV"	=> "Wolverine",
		"MW"	=> "Weasel",
		"MaB"	=> "Wombat",
		"MaK"	=> "Kangaroo",
		"MaO"	=> "Koala",
		"MaP"	=> "Possum",
		"MaT"	=> "Tasmanian devil",
		"MaW"	=> "Wallaby",
		"MoP"	=> "Platypus",
		"PR"	=> "Raccoon",
		"PiL"	=> "Sea lion",
		"PiS"	=> "Seal",
		"PiW"	=> "Walrus",
		"PrL"	=> "Lemur",
		"RB"	=> "Beaver",
		"RG"	=> "Grey squirrel",
		"RM"	=> "Mouse",
		"RP"	=> "Porcupine",
		"RR"	=> "Rat",
		"RS"	=> "Red squirrel",
		"RU"	=> "Muskrat",
		"UA"	=> "Polar bear",
		"UB"	=> "Black bear",
		"UG"	=> "Grizzly bear/brown bear/Kodiak bear",
		"UP"	=> "Giant panda",
		"VC"	=> "Civet",
		"XA"	=> "Gargoyle",
		"XC"	=> "Manticore",
		"XG"	=> "Gryphon",
		"XH"	=> "Hippogriff",
		"XM"	=> "Mermaid/merman",
	);
	defined $families{$fam} or return 0;
	out "$families{$fam}";
	my $fs = "$fam$spec";
	if (defined $species{$fs}) {
		out ": $species{$fs}";
	} elsif ($spec ne "") {
		out ": Unknown species $fs";
	}
	($name =~ /^\[(.*)\]/) && out ": $1" if defined $name;

	my %mods = (
		"c" => "Cyborg",
		"f" => "\"Funny animal\" (toon)",
		"h" => "Were-human",
		"m" => "With magical powers",
		"p" => "Polymorph (the given species is my normal form, but I can change shape)",
		"s" => "With psychic powers",
		"t" => "Taur",
		"u" => "Unicorn",
		"w" => "Winged",
	);
	if ($mod ne "") {
		out ", ";
		out join ", ", map { defined($mods{$_}) ? $mods{$_} : "Unknown modifier $_" } split //, $mod;
	}

	my %sizes = (
		"1" => "Basically human, with minor furry features (perhaps eyes, nose, ears, claws, some fur, etc)",
		"2" => "Humanoid, with significant furry features (muzzle, tail, etc); this includes centaurs and mer-people",
		"3" => "Anthropomorphic animal",
		"4" => "Equally comfortable on two or four legs",
		"5" => "Animal shape, with some unusual features (perhaps hands, speech, etc); this includes most dragons, gryphons, etc",
		"6" => "Normal animal shape",
	);
	$sizes{"4"} = "Equally comfortable on four or six legs" if $mod =~ /t/;
	out ", " . (defined($sizes{$size}) ? $sizes{$size} : "Unknown size code $size") if $size ne "";

	my %rels = (
		"a" => "just a general \"alter ego\"",
		"c" => "a costume I wear",
		"d" => "someone to draw pictures of",
		"f" => "my imaginary friend",
		"m" => "my online MU* character (FurryMuck etc)",
		"r" => "my role-playing game character",
		"s" => "my guardian spirit or totem",
		"w" => "someone to write stories about",
	);
	if ($rel ne "") {
		out ". This furry is ";
		out join ", ", map { defined($rels{$_}) ? $rels{$_} : "Unknown relifier $_" } split //, $rel;
	}

	return 1;
}

my @cats;

BEGIN { @cats = (
	"=== My furry side ===",
	"F" => {
		"#name" => "Furry species",
		"#decode" => \&decode_species,
		},
	"A" => {
		"#name" => "Art",
		"++++" => "Art is my life",
		"+++" => "My art appears regularly in zines and elsewhere, and people ask me to contribute to their sketchbooks",
		"++" => "I have pictures in reasonably well-known zines and/or Web sites",
		"+" => "I draw regularly, and someone once said something that could possibly be construed as a compliment",
		"" => "I've shown one or two of my pictures to others, and they didn't actually throw up",
		"-" => "Tried a few sketches in the privacy of my own home",
		"--" => "Never tried",
		"---" => "Never tried, never will",
		},
	"C" => {
		"#name" => "Conventions",
		"+++" => "Been to lots of cons, organised at least one",
		"++" => "I'm a regular con-goer, and I've occasionally lent a paw",
		"+" => "I've been to several, and plan to go to many more",
		"" => "I've been to one",
		"-" => "Never been to one, but may do so in future",
		"--" => "Not interested",
		"---" => "I wouldn't go near one of those places if you paid me",
		"*" => "Haven't been to a furry con, but I have been to an SF con",
		"**" => "Haven't been to a furry con, but I've helped organise an SF con",
		},
	"D" => {
		"#name" => "Dressing up",
		"++++" => "I've made plans to be buried in a fursuit",
		"+++" => "I'll wear a fursuit at any opportunity (where costumes are not expected)",
		"++" => "I'll wear a fursuit at cons/sporting events (where costumes are uncommon)",
		"+" => "I'll wear a fursuit for Halloween/masquerades (where costumes are expected)",
		"" => "I might wear a fursuit",
		"-" => "I'd wear a fursuit if I had to",
		"--" => "You must be kidding, I'd die first",
		},
	"H" => {
		"#name" => "Hugs",
		"+++" => "If it moves, I'll hug it (if it doesn't move, I'll hug it until it moves)",
		"++" => "I'll hug anyone I know, given a faint excuse",
		"+" => "I'll accept hugs, and maybe even give the occasional one",
		"" => "Well, OK, you can hug me if you really want to",
		"-" => "Please don't, unless we know each other very well",
		"--" => "No way",
		"---" => "Argh! Don't touch me, you pervert",
		},
	"M" => {
		"#name" => "Mucking and mudding",
		"++++" => "I'm a Wizard, tremble before my might",
		"+++" => "Someday I may get around to trying Real Life, but it's not high on my agenda",
		"++" => "I've got characters on several MU*s, and frequently get frustrated when commands don't work in RL",
		"+" => "I'm a regular on at least one furry MU*",
		"" => "Tried them once or twice, may do it again sometime",
		"-" => "Never been able to dredge up enough interest (or time) to try it",
		"--" => "Those things are for weenies",
		"---" => "You guys are pathetic, get a life",
		},
	"P" => {
		"#name" => "Plush critters",
		"++++" => "Where'd my plushie go? Oh, it's buried under my stuffies",
		"+++" => "I collect every one I find of a certain species (or two, or three...)",
		"++" => "I've got a collection of several favourites",
		"+" => "I have been known to cuddle a few",
		"" => "I like them, they sit on my shelves collecting dust",
		"-" => "No thanks, they'll sit on my shelves collecting dust",
		"--" => "Kid stuff, I'd sooner hug a pincushion",
		},
	"R" => {
		"#name" => "Realism vs tooniness",
		"+++" => "What do you mean it isn't a photograph?",
		"++" => "Figments of the imagination have anatomies too, you know",
		"+" => "I like both, but prefer realistic furries",
		"" => "No particular preference",
		"-" => "I like both, but prefer toons",
		"--" => "The toonier the better",
		"---" => "If it's not Super-Deformed I don't want to know about it",
		},
	"T" => {
		"#name" => "Transformation",
		"++++" => "Never mind the fine print, where do I sign?",
		"+++" => "Definitely! (as long as I get to choose the species)",
		"++" => "Yes, if it's reversible",
		"+" => "Probably, as long as I wasn't the first guinea pig (or whatever...)",
		"" => "I'd have to think about it carefully",
		"-" => "Not personally",
		"--" => "What a horrible idea",
		},
	"W" => {
		"#name" => "Writing",
		"++++" => "I've out-sold Anne McCaffrey with my furry books",
		"+++" => "I've sold a furry book",
		"++" => "I've sold a furry story to a real magazine (duck)",
		"+" => "I've sold furry stuff to fanzines",
		"****" => "I've out-sold Anne McCaffrey with my non-furry books",
		"***" => "I've sold a non-furry book",
		"**" => "I've sold a non-furry story to a real magazine (duck)",
		"*" => "I've sold non-furry stuff to fanzines",
		"" => "I've written a story that somebody else has read",
		"-" => "I have these scribblings but nobody is ever going to see them",
		"--" => "Never written a word of fiction (Inland Revenue forms excepted)",
		"---" => "Illiterate",
		},
	"Z" => {
		"#name" => "Zines",
		"++++" => "Publisher/editor/other staff on a professional furry-related publication",
		"+++" => "I work on a regularly-published amateur zine, or I'm a frequently-published author/artist",
		"++" => "I've been published, or directly involved in a publication, at least once",
		"+" => "I have a good collection, and buy at least one title regularly",
		"" => "I have a few furry zines",
		"-" => "Not really interested",
		"--" => "Comics are for kids",
		},
	"S" => {
		"#name" => "Furry sex",
		"+++" => "What do you mean there are other things to do?",
		"++" => "Ready, willing, and able",
		"+" => "I've had TinySex",
		"" => "Never had TinySex, but wouldn't rule it out",
		"-" => "Celibate",
		"--" => "You people are sick!",
		"f:" => "Female",
		"h:" => "Hermaphrodite",
		"m:" => "Male",
		"p:" => "Polymorph (can change sex at will)",
		},
	"=== My human side ===",
	"RL" => {
		"#name" => "What I do in Real Life",
		"A" => "Art",
		"BM" => "Business/management",
		"C" => "Craft",
		"CI" => "Construction industry",
		"CT" => "Computers/information technology",
		"E" => "Engineering",
		"ET" => "Education/teaching",
		"F" => "Farming",
		"FB" => "Finance/banking",
		"GP" => "Government/public service",
		"L" => "Law",
		"LW" => "Literature/writing",
		"M" => "Music",
		"MA" => "Military/armed forces",
		"MC" => "Media/communications",
		"MH" => "Medicine (human)",
		"RB" => "Retail business",
		"S" => "Science",
		"TH" => "Theatre",
		"TI" => "Transport industry",
		"VM" => "Veterinary medicine",
		"AT" => "I'm a furry of all trades (\"Specialisation is for insects!\" -- Robert Heinlein)",
		"U" => "Undecided (generally used by young students who haven't picked a major yet)",
		"-" => "No qualifications, no job, no complaints",
		"#decode" => sub {
			my ($c, $val) = @_;
			if ($val =~ /^(.*)\*$/ && $c->{$1}) {
				out "Trained in $c->{$1}";
				return 1;
			}
			return 0;
		}
		},
	"a" => {
		"#name" => "Age",
		"++++" => "60+ years",
		"+++" => "50-59 years",
		"++" => "40-49 years",
		"+" => "30-39 years",
		"" => "20-29 years",
		"-" => "10-19 years",
		"--" => "Under 10 years",
		"#decode" => sub {
			my ($c, $val) = @_;
			($val =~ /^\d+$/) and out "$val years" and return 1;
			return 0;
			}
		},
	"c" => {
		"#name" => "Computers",
		"++++" => "I'll be first in line to get a cybernetic interface installed in my skull!",
		"+++" => "Hey, if there was anything else to life, there'd be a newsgroup about it",
		"++" => "Computers are a large part of my life; I spend time every day in front of one; I've tried my hand at programming",
		"+" => "Computers are fun; I can use some software without resorting to the manual; I play a mean game of [insert favourite game]",
		"" => "Computers are just a tool; I use one when it serves my purpose",
		"-" => "I'm nervous of anything more complicated than my microwave",
		"--" => "Where's the \"on\" switch? Better yet, where's the \"off\" switch?",
		"---" => "They're taking over the world! Smash the machines! Up the Luddites!",
		"#decode" => sub {
			my ($c, $val) = @_;
			my ($px, $rest) = ($val =~ /^([a-z]+)([^a-z]*)$/) or return 0;
			defined $c->{$rest} or return 0;
			out "$c->{$rest}; I use ";
			my %tab = (
				"a" => "Amiga",
				"b" => "BSD Unix",
				"d" => "MS-DOS",
				"l" => "Linux",
				"m" => "Macintosh",
				"n" => "Windows 95/NT",
				"o" => "OS/2",
				"u" => "Unix (commercial)",
				"v" => "VMS",
				"w" => "Windows 3.x",
				);
			out join ", ", map { defined($tab{$_}) ? $tab{$_} : "Unknown flag $_" } split //, $px;
			return 1;
			}
		},
	"d" => {
		"#name" => "Doom, Quake, etc.",
		"++++" => "I work for ID, bow down before me",
		"+++" => "I can solve every level in Nightmare mode with my eyes shut; I crank out new WAD files daily",
		"++" => "I've got pretty good at it; I can get through most levels easily; I've downloaded and played other WADs",
		"+" => "It's a fun, action game that is a nice diversion on a lazy afternoon",
		"" => "I've played the game and I'm pretty indifferent",
		"-" => "I've played the game and really didn't think it was all that impressive",
		"--" => "I miss Zork",
		"---" => "All this violence is sickening; there ought to be a law",
		},
	"e" => {
		"#name" => "Education",
		"++++" => "Doctorate, or the equivalent",
		"+++" => "Master's degree, or the equivalent",
		"++" => "Bachelor's degree, or the equivalent",
		"+" => "Some tertiary education",
		"" => "Finished high school",
		"-" => "Haven't finished high school",
		"--" => "Haven't started high school",
		"*" => "Learned everything there is to know about life from The Hitch-Hiker's Guide to the Galaxy, my collection of Omaha comics, and late-night reruns of Star Trek and The Prisoner",
		"**" => "Graduate degree from the School of Hard Knocks",
		},
	"f" => {
		"#name" => "Real life furriness factor",
		"++++" => "I am not a human, I am a furry",
		"+++" => "I've been known to bark at people I don't know to greet them",
		"++" => "I've been known to bark at friends to greet them",
		"+" => "I make frequent jokes, show friends my furry art collection",
		"" => "I make an occasional reference to my furriness",
		"-" => "I only tell close friends",
		"--" => "I tell nobody",
		},
	"h" => {
		"#name" => "Housing",
		"++++" => "Married ... with children",
		"+++" => "Married, or shacked up with your SO on a long-term basis",
		"++" => "Living with one or more fellow furries",
		"+" => "Living with one or more people who know nothing about furriness",
		"" => "Living alone, other furries come to visit",
		"-" => "Living alone, get out once a week to buy food, all surfaces covered in computers and/or zines",
		"--" => "Living in a cave with 47 computers and a T1 line",
		"*" => "I'm still stuck living with my parents",
		"**" => "I'm not sure where I live any more; my workplace/lab seems like home to me",
		"***" => "Homeless",
		},
	"i" => {
		"#name" => "Internet",
		"+++" => "I'm a Webmaster/site administrator",
		"++" => "I spend most of my spare time surfing the Web, and read any newsgroup that catches my interest",
		"+" => "I browse the Web regularly, and read a handful of newgroups",
		"" => "I have a browser and a connection, and even use them occasionally",
		"-" => "Not connected yet",
		"--" => "The Internet sucks; it's all just a flash in the pan anyway",
		"---" => "It's a dangerous, subversive, perverted abomination that needs to be banned before people stop voting for me",
		"w:" => "I have my own webpage",
		"wf:" => "I have my own furry webpage",
		},
	"j" => {
		"#name" => "Anime (Japanese animation)",
		"++++" => "I am a writer/artist/seyeiuu",
		"+++" => "I own every episode of every series ever made",
		"++" => "I watch it in all my spare time (furry fandom is, of course, not spare time)",
		"+" => "Others know I'm an Anime fan",
		"" => "Seen it, might think about seeing it again some time",
		"-" => "Haven't seen it, but would be interested when I get the time",
		"--" => "What's Anime?",
		"---" => "There should be a law against anything that cute",
		"*" => "I'll watch it, but only if it's all in English",
		},
	"p" => {
		"#name" => "Pets",
		"+++" => "I have a vast household of assorted furry/scaly/feathery creatures, and my life is organised for their benefit",
		"++" => "Several pets",
		"+" => "Two or three conventional pets (cats, dogs, etc), or one fairly exotic one",
		"" => "One pet of a fairly conventional type (cat, dog, etc)",
		"-" => "No pets currently, but I may enrich my life in the future",
		"--" => "I don't have any pets; my lifestyle/household/schedule is complicated enough already",
		"---" => "I wouldn't have the things in the house",
		"*" => "I'd like to have pets, but my landlord/parents/flatmates won't allow them",
		"**" => "Sorry, I'm allergic to animals",
		},
	"s" => {
		"#name" => "Human sex",
		"m:" => "Male",
		"f:" => "Female",
		"+++" => "There's more to life? Where is it, and what equipment do you need?",
		"++" => "I was once referred to as \"easy\", but I have no idea where that might have come from",
		"+" => "I've had real, live sex",
		"" => "I've had sex ... oh, you mean with someone else?",
		"-" => "Not having sex by choice",
		"--" => "Not having sex because I can't get any",
		"---" => "Not having sex because I'm a nun/priest",
		"*" => "I'm married, so I can get it whenever I want (well, that's the theory, anyway)",
		"**" => "I have a few little rug rats to prove I've been there (but with kids around, who has time for sex?)",
		}
);
}

sub get_parsing_table($) {
	my $ver = shift @_;
	if (!defined($ver) || $ver =~ /1\./) {
		return @cats;
	} else {
		return ();
	}
}

sub parse_code($) {
	my $x = shift @_;
	if (my ($mod, $cat, $val) = ($x =~ /(!?)([A-QS-Za-z]|RL?)(.*)/)) {
		return ($mod, $cat, $val);
	}
	return undef;
}

# Go!
main();

1;
